/* describe.c -	Set or list a descriptive comment for a file
 *
 * Copyright (c) 1991 Tim Cook.
 * Non-profit distribution allowed.  See README for details.
 */

static char rcsid[] = "$Header: describe.c 1.0 91/03/22 $" ;

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/dir.h>
#include <sys/file.h>
#include <sys/stat.h>
#ifdef NDBM
#include <ndbm.h>
#else
#if defined(sequent) && defined(NULL)
#undef NULL	/* On the Sequent, dbm.h #define's NULL */
#endif
#include <dbm.h>
#endif
#include <vclib.h>

#ifdef sequent
#define strrchr	rindex
#endif

#define NULL_CP	((char *) 0)
#define EOS	'\0'

static char *program_name ;
#ifdef NDBM
static DBM *desc_database ;
#endif

static void usage () ;
static int init_desc_file () ;
static void set_description () ;
static void set_from_file () ;
static void delete_description () ;
static void list_descriptions () ;


int main (argc, argv)
   int argc ;
   char **argv ;
{
   extern char *optarg ;
   extern int optind, opterr ;
   char directory[MAXNAMLEN+1] ;
   char file_name[MAXNAMLEN+1] ;
   int delete_desc = FALSE ;
   struct stat status ;
   int option ;
   char *p ;

   program_name = argv[0] ;
   while ((option = getopt (argc, argv, "dls:")) != EOF) {
      switch (option) {
	 case 's':	/* Set descriptions from file */
	    if (delete_desc)
	       usage () ;
	    if (optind >= argc)
	       p = "." ;
	    else
	       p = argv[optind] ;
	    init_desc_file (p, FALSE) ;
	    set_from_file (optarg, p) ;
#ifdef NDBM
	    dbm_close (desc_database) ;
#else
	    dbmclose () ;
#endif
	    exit (0) ;
	    break ;
         case 'l':	/* List directory */
	    if (delete_desc)
	       usage () ;
	    if (optind >= argc)
	       p = "." ;
	    else
	       p = argv[optind] ;
	    if (! init_desc_file (p, TRUE)) {
	       fprintf (stderr, "%s: no descriptions for directory %s\n",
		  program_name, p) ;
	       exit (2) ; }
	    list_descriptions () ;
#ifdef NDBM
	    dbm_close (desc_database) ;
#else
	    dbmclose () ;
#endif
	    exit (0) ;
	    break ;
         case 'd':	/* Delete description(s) */
	    delete_desc = TRUE ;
	    break ;
         default:
	    usage () ; } }

   if (delete_desc) {
      if (optind >= argc)
	 usage () ;
      for ( ; optind < argc ; optind++) {
	 split_pathname (argv[optind], directory, file_name) ;
	 if (stat (argv[optind], &status) == -1) {
	    perror2 (program_name, argv[optind]) ; }
	 else {
	    init_desc_file (directory, FALSE) ;
	    delete_description (file_name, status.st_ino) ; } } }
   else {
      if (argc != 3)
	 usage () ;

      /* Set description of specified file */
      split_pathname (argv[1], directory, file_name) ;
      if (stat (argv[1], &status) == -1) {
	 perror2 (program_name, argv[1]) ;
	 exit (2) ; }
      init_desc_file (directory, FALSE) ;
      set_description (file_name, status.st_ino, argv[2]) ; }

#ifdef NDBM
   dbm_close (desc_database) ;
#else
   dbmclose () ;
#endif
   }


static
void usage ()
{
   fprintf (stderr, "\
usage: %s <file> <description>\n\
   or: %s -s <descriptions-file> [<directory>]\n\
   or: %s -d <file> ...\n\
   or: %s -l [<directory>]\n",
      program_name, program_name, program_name, program_name) ;
   exit (1) ;
   }


static
void set_from_file (file, directory)
   char *file ;
   char *directory ;
{
   FILE *descriptions ;
   char file_name[MAXNAMLEN+1] ;
   char buffer[1024] ;
   char description[256] ;
   struct stat status ;
   int n ;

   if (strcmp (file, "-") == 0)
      descriptions = stdin ;
   else {
      if ((descriptions = fopen (file, "r")) == (FILE *) NULL) {
	 perror2 (program_name, file) ;
	 exit (2) ; } }
   if (chdir (directory) == -1) {
      perror2 (program_name, directory) ;
      exit (2) ; }
   while (! feof (descriptions)) {
      if (fgets (buffer, sizeof (buffer), descriptions) != NULL_CP) {
	 n = sscanf (buffer, "\"%[^\"]\" %[^\n]\n", file_name, description) ;
	 if (n < 2)
	    n = sscanf (buffer, "%s %[^\n]\n", file_name, description) ;
	 if (n == 2 && file_name[0] != '#') {
	    if (stat (file_name, &status) == -1) {
	       perror2 (program_name, file_name) ; }
	    else {
	       set_description (file_name, status.st_ino, description) ; } } }
      }
   fclose (descriptions) ;
   }


#define uninitialised	0
#define not_found	1
#define initialised	2

static int desc_state = uninitialised ;

static
int init_desc_file (file_name, read_only)
   char *file_name ;
   int read_only ;
{
   static char desc_file[MAXNAMLEN+1] ;
   char new_desc_file[MAXNAMLEN+1] ;
   char name[MAXNAMLEN+1] ;
   char *p ;
#ifdef NDBM
   int open_flags ;
#else
   struct stat status ;
   int fd ;
#endif

   strcpy (new_desc_file, file_name) ;
   if (desc_state == uninitialised || strcmp (desc_file, new_desc_file) != 0) {

      /* Changing desc_file */

      strcpy (desc_file, new_desc_file) ;
      if (*new_desc_file != EOS)
	 strcat (new_desc_file, "/") ;
      strcat (new_desc_file, ".desc.pag") ;
      p = strrchr (new_desc_file, '.') ;

#ifdef NDBM

      /* By crikey, it's easier this way! */

      *p = EOS ;

      if (read_only)
	 open_flags = O_RDONLY ;
      else
	 open_flags = O_RDWR | O_CREAT ;
      if ((desc_database = dbm_open (new_desc_file, open_flags, 0666))
               == (DBM *) NULL) {
	 desc_state = not_found ;
	 return FALSE && (! read_only) ; }
      desc_state = initialised ;

#else	/* DBM */

      if (stat (new_desc_file, &status) != -1) {	/* File exists */
	 *p = EOS ;
	 if (dbminit (new_desc_file) < 0) {
	    desc_state = not_found ;
	    return FALSE ; }
	 else
	    desc_state = initialised ; }
      else {
	 if (read_only) {
	    desc_state = not_found ;
	    return FALSE ; }		/* Signal that there isn't one */
	 else {
	    /* Create .desc.(pag|dir) files */
	    if ((fd = open (new_desc_file, O_CREAT | O_EXCL | O_RDWR, 0666))
		< 0) {
	       perror2 (program_name, new_desc_file) ;
	       exit (2) ; }
	    close (fd) ;
	    strcpy (p, ".dir") ;
	    if ((fd = open (new_desc_file, O_CREAT | O_EXCL | O_RDWR, 0666))
		< 0) {
	       perror2 (program_name, new_desc_file) ;
	       exit (2) ; }
	    close (fd) ;
	    *p = EOS ;

	    /* Start up DBM */
	    if (dbminit (new_desc_file) < 0)
	       desc_state = not_found ;
	    else
	       desc_state = initialised ; } }
#endif	/* NDBM */
      }
   return TRUE ;
   }


static
void set_description (file_name, inode, description)
   char *file_name ;
   ino_t inode ;
   char *description ;
{
   datum key, content ;

   /* Store the description, indexed by the file name */
   key.dptr = file_name ;
   key.dsize = strlen (file_name) ;
   if (key.dsize == sizeof (ino_t))
      key.dsize++ ;
   content.dptr = description ;
   content.dsize = strlen (description) + 1 ;
#ifdef NDBM
   if (dbm_store (desc_database, key, content, DBM_REPLACE) < 0)
#else
   if (store (key, content) < 0)
#endif
   {
      perror2 (program_name, file_name) ;
      exit (2) ; }

   /* Store the file name, indexed by the inode number */
   key.dptr = (char *) &inode ;
   key.dsize = sizeof (ino_t) ;
   content.dptr = file_name ;
   content.dsize = strlen (file_name) ;
#ifdef NDBM
   if (dbm_store (desc_database, key, content, DBM_REPLACE) < 0)
#else
   if (store (key, content) < 0)
#endif
   {
      perror2 (program_name, file_name) ;
      exit (2) ; }
   }


static
void delete_description (file_name, inode)
   char *file_name ;
   ino_t inode ;
{
   datum key ;

   key.dptr = (char *) &inode ;
   key.dsize = sizeof (ino_t) ;
#ifdef NDBM
   dbm_delete (desc_database, key) ;
#else
   delete (key) ;
#endif
   key.dptr = file_name ;
   key.dsize = strlen (file_name) ;
   if (key.dsize == sizeof (ino_t))
      key.dsize++ ;
#ifdef NDBM
   dbm_delete (desc_database, key) ;
#else
   delete (key) ;
#endif
   }


static
void list_descriptions ()
{
   datum key, last_key, content ;
   char file_name[MAXNAMLEN+1] ;
   int file_name_length ;
   int gap ;

#ifdef NDBM
   for (key = dbm_firstkey (desc_database) ; key.dptr != NULL_CP ;
	 key = dbm_nextkey (desc_database))
#else
   for (key = firstkey () ; key.dptr != NULL_CP ; key = nextkey (last_key))
#endif
   {
      last_key = key ;
      if (key.dsize != sizeof (ino_t)) {
#ifdef NDBM
	 content = dbm_fetch (desc_database, key) ;
#else
	 content = fetch (key) ;
#endif
	 file_name_length = key.dsize ;
	 if (file_name_length == sizeof (ino_t) + 1
	     && key.dptr[sizeof(ino_t)] == EOS)
	    file_name_length-- ;
	 strncpy (file_name, key.dptr, file_name_length) ;
	 file_name[file_name_length] = EOS ;
	 if (strpbrk (file_name, " \t") != NULL_CP) {
	    printc ('"') ;
	    print (file_name) ;
	    printc ('"') ;
	    gap = 18 - file_name_length ; }
	 else {
	    print (file_name) ;
	    gap = 20 - file_name_length ; }
	 printc (' ') ;		/* We need at least one space */
	 while (gap-- > 0)
	    printc (' ') ;
	 print (content.dptr) ;
         printc ('\n') ; } }
   }
